@@ -1,5 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
+ * Copyright (C) 2003-2004 Imendio AB
  * Copyright (C) 2001-2003 CodeFactory AB
  * Copyright (C) 2001-2003 Richard Hult <richard@imendio.com>
  * Copyright (C) 2001-2002 Mikael Hallendal <micke@imendio.com>
@@ -23,12 +24,8 @@
 
 #include <config.h>
 #include <string.h>
-
-#include <gsf/gsf-input-memory.h>
-#include <gsf/gsf-input-stdio.h>
-
 #include "mrp-error.h"
-#include "mrp-intl.h"
+#include <glib/gi18n.h>
 #include "mrp-marshal.h"
 #include "mrp-task-manager.h"
 #include "mrp-storage-module-factory.h"
@@ -414,7 +411,6 @@ static void
 project_init (MrpProject *project)
 {
 	MrpProjectPriv *priv;
-	MrpProperty    *property;
 	MrpCalendar    *calendar;
 	
 	project->priv = g_new0 (MrpProjectPriv, 1);
@@ -447,7 +443,7 @@ project_init (MrpProject *project)
 		(GDestroyNotify) mrp_day_unref);
 	
         /* Move these into the cost plugin */
-	property = mrp_property_new ("cost", 
+/*	property = mrp_property_new ("cost", 
 				     MRP_PROPERTY_TYPE_COST,
 				     "Cost",
 				     "standard cost for a resource",
@@ -456,7 +452,7 @@ project_init (MrpProject *project)
 				  MRP_TYPE_RESOURCE,
 				  property,
                                   TRUE);
-
+*/
 	/* FIXME: Add back this when it's used. */
 /*	property = mrp_property_new ("cost_overtime", 
 	MRP_PROPERTY_TYPE_COST,
@@ -683,9 +679,10 @@ gboolean
 mrp_project_load (MrpProject *project, const gchar *uri, GError **error)
 {
 	MrpProjectPriv *priv;
-	GsfInput       *input;
 	GList          *l;
 	MrpCalendar    *old_default_calendar;
+	gchar          *file_str;
+	size_t          len;
 	
 	g_return_val_if_fail (MRP_IS_PROJECT (project), FALSE);
 	g_return_val_if_fail (uri != NULL, FALSE);
@@ -702,25 +699,38 @@ mrp_project_load (MrpProject *project, const gchar *uri, GError **error)
 	 */
 	old_default_calendar = priv->calendar;
 
- 	input = GSF_INPUT (gsf_input_mmap_new (uri, NULL));
-	
- 	if (!input) {
-		input = GSF_INPUT (gsf_input_stdio_new (uri, error));
-	}
 
-	if (!input) {
+	len = strlen (uri);
+
+	/* Get a local file from the uri. */
+	if (len > 3 && !strstr (uri, ":/")) {
+		/* No protocol. */
+	} else {
+		if (len > 7 && !strncmp (uri, "file:/", 6)) {
+			/* Naively strip method. */
+			uri = uri + 7;
+		} else {
+			g_set_error (error, 
+				     MRP_ERROR,
+				     MRP_ERROR_INVALID_URI,
+				     _("Invalid URI: '%s'"),
+				     uri);
+			
+			return FALSE;
+		}
+	}
+	
+	if (!g_file_get_contents (uri, &file_str, NULL, error)) {
 		return FALSE;
 	}
-
+	
 	mrp_task_manager_set_block_scheduling (priv->task_manager, TRUE);
 	
 	l = imrp_application_get_all_file_readers (priv->app);
 	for (; l; l = l->next) {
 		MrpFileReader *reader = l->data;
 
-		if (mrp_file_reader_read (reader, input, project, error)) {
-			g_object_unref (input);
-			
+		if (mrp_file_reader_read_string (reader, file_str, project, error)) {
 			g_signal_emit (project, signals[LOADED], 0, NULL);
 			imrp_project_set_needs_saving (project, FALSE);
 
@@ -731,8 +741,6 @@ mrp_project_load (MrpProject *project, const gchar *uri, GError **error)
 			mrp_calendar_remove (old_default_calendar);
 
 			mrp_task_manager_set_block_scheduling (priv->task_manager, FALSE);
-
-			/* FIXME: See bug #416. */
 			imrp_project_set_needs_saving (project, FALSE);
 			
 			return TRUE;
@@ -741,8 +749,6 @@ mrp_project_load (MrpProject *project, const gchar *uri, GError **error)
 
 	mrp_task_manager_set_block_scheduling (priv->task_manager, FALSE);
 
-	g_object_unref (input);
-
 	g_set_error (error, 
 		     MRP_ERROR,
 		     MRP_ERROR_NO_FILE_MODULE,
@@ -767,7 +773,7 @@ project_do_save (MrpProject   *project,
 		if (!project_set_storage (project, "sql")) {
 			g_set_error (error, MRP_ERROR,
 				     MRP_ERROR_NO_FILE_MODULE,
-				     _("No support for SQL storage built into this version of MrProject."));
+				     _("No support for SQL storage built into this version of Planner."));
 			return FALSE;
 		}
 	} else {
@@ -843,13 +849,31 @@ mrp_project_save_as (MrpProject   *project,
 		     GError      **error)
 {
 	MrpProjectPriv *priv;
+	gboolean        is_sql;
+	gchar          *real_uri;
 	
 	g_return_val_if_fail (MRP_IS_PROJECT (project), FALSE);
 	g_return_val_if_fail (uri != NULL && uri[0] != '\0', FALSE);
 	
 	priv = project->priv;
 
-	if (!project_do_save (project, uri, force, error)) {
+	if (strncmp (uri, "sql://", 6) == 0) {
+		is_sql = TRUE;
+
+		real_uri = g_strdup (uri);
+	} else {
+		is_sql = FALSE;
+
+		/* Hack for now. */
+		if (!strstr (uri, ".mrproject") && !strstr (uri, ".planner")) {
+			real_uri = g_strconcat (uri, ".planner", NULL);
+		} else {
+			real_uri = g_strdup (uri);
+		}
+	}
+	
+	if (!project_do_save (project, real_uri, force, error)) {
+		g_free (real_uri);
 		return FALSE;
 	}
 
@@ -858,15 +882,17 @@ mrp_project_save_as (MrpProject   *project,
 	/* A small hack for now: check if it's SQL and update the URI to include
 	 * the newly assigned id.
 	 */
-	if (strncmp (uri, "sql://", 6) == 0) {
+	if (is_sql) {
 		gchar *new_uri;
 		
 		new_uri = g_object_get_data (G_OBJECT (priv->primary_storage), "uri");
 		priv->uri = g_strdup (new_uri);
 	} else {
-		priv->uri = g_strdup (uri);
+		priv->uri = g_strdup (real_uri);
 	}
 	
+	g_free (real_uri);
+	
 	imrp_project_set_needs_saving (project, FALSE);
 	
 	return TRUE;
@@ -1031,7 +1057,7 @@ project_load_from_sql (MrpProject   *project,
 	if (!project_set_storage (project, "sql")) {
 		g_set_error (error, MRP_ERROR,
 			     MRP_ERROR_NO_FILE_MODULE,
-			     _("No support for SQL storage built into this version of MrProject."));
+			     _("No support for SQL storage built into this version of Planner."));
 		return FALSE;
 	}
 
@@ -1097,7 +1123,7 @@ project_set_storage (MrpProject  *project,
 
 /**
  * mrp_project_close:
- * @project: an #MrProject
+ * @project: an #MrpProject
  * 
  * Closes a project.
  **/
@@ -1210,8 +1236,10 @@ mrp_project_add_resource (MrpProject *project, MrpResource *resource)
 		g_object_set (resource, "type", MRP_RESOURCE_TYPE_WORK, NULL);
 	}
 
+	
 	project_connect_object (MRP_OBJECT (resource), project);
 	
+
 	g_signal_emit (project, signals[RESOURCE_ADDED], 0, resource);
 
 	imrp_project_set_needs_saving (project, TRUE);
@@ -1407,6 +1435,21 @@ mrp_project_get_uri (MrpProject *project)
 	return project->priv->uri;
 }
 
+/**
+ * mrp_project_set_uri:
+ * @project: an #MrpProject
+ * 
+ * Sets the URI on @project.
+ **/
+void    
+mrp_project_set_uri (MrpProject *project, const gchar *uri)
+{
+	g_return_if_fail (MRP_IS_PROJECT (project));
+
+	g_free (project->priv->uri);
+	project->priv->uri = g_strdup (uri);
+}
+
 /**
  * mrp_project_get_project_start:
  * @project: an #MrpProject
@@ -1471,7 +1514,7 @@ imrp_project_add_calendar_day (MrpProject *project, MrpDay *day)
 	g_hash_table_insert (priv->day_types,
 			     GINT_TO_POINTER (mrp_day_get_id (day)), 
 			     mrp_day_ref (day));
-	
+
 	g_signal_emit (project,
 		       signals[DAY_ADDED],
 		       0,
@@ -1482,6 +1525,30 @@ imrp_project_add_calendar_day (MrpProject *project, MrpDay *day)
 	return TRUE;
 }
 
+/**
+ * mrp_project_get_calendar_day_by_id:
+ * @project: an #MrpProject
+ * 
+ * Semi-private function. You most likely won't need this outside of
+ * Planner. Returns the day type associated with @id.
+ * 
+ * Return value: A day type.
+ **/
+MrpDay *
+mrp_project_get_calendar_day_by_id (MrpProject *project, gint id)
+{
+	MrpProjectPriv *priv;
+	MrpDay         *day;
+	
+	g_return_val_if_fail (MRP_IS_PROJECT (project), NULL);
+
+	priv = project->priv;
+	
+	day = g_hash_table_lookup (priv->day_types, GINT_TO_POINTER (id));
+
+	return day;
+}
+
 static void
 foreach_day_add_to_list (gpointer key, MrpDay *day, GList **list)
 {
@@ -1843,7 +1910,7 @@ mrp_project_reschedule (MrpProject *project)
 {
 	g_return_if_fail (MRP_IS_PROJECT (project));
 
-	mrp_task_manager_recalc (project->priv->task_manager, FALSE);
+	mrp_task_manager_recalc (project->priv->task_manager, TRUE);
 }
 
 /**
@@ -2215,3 +2282,27 @@ project_set_calendar (MrpProject  *project,
 	mrp_task_manager_recalc (priv->task_manager, TRUE);
 }
 
+void
+mrp_project_set_block_scheduling (MrpProject *project, gboolean block)
+{
+	MrpProjectPriv *priv;
+
+	g_return_if_fail (MRP_IS_PROJECT (project));
+
+	priv = project->priv;
+
+	mrp_task_manager_set_block_scheduling (priv->task_manager, block);
+}
+
+gboolean
+mrp_project_get_block_scheduling (MrpProject *project)
+{
+	MrpProjectPriv *priv;
+
+	g_return_val_if_fail (MRP_IS_PROJECT (project), FALSE);
+
+	priv = project->priv;
+	
+	return mrp_task_manager_get_block_scheduling (priv->task_manager);
+}
+
