@@ -1,5 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
+ * Copyright (C) 2004-2005 Imendio AB
  * Copyright (C) 2001-2002 CodeFactory AB
  * Copyright (C) 2001-2002 Richard Hult <richard@imendio.com>
  * Copyright (C) 2001-2002 Mikael Hallendal <micke@imendio.com>
@@ -23,7 +24,7 @@
 #include <config.h>
 #include <string.h>
 #include "mrp-marshal.h"
-#include "mrp-intl.h"
+#include <glib/gi18n.h>
 #include "mrp-private.h"
 #include "mrp-resource.h"
 #include "mrp-error.h"
@@ -64,8 +65,9 @@ struct _MrpTaskPriv {
 
 	/* Used for topological order sorting. */
 	guint             visited : 1;
-	GNode            *sorted_node;
 
+	MrpTaskGraphNode *graph_node;
+	
 	/* FIXME: This might be a mistake... I can't think of any other types,
 	 * besides milestone and normal. Should we have a boolean instead,
 	 * is_milestone? That or a flags type variable.
@@ -122,6 +124,9 @@ struct _MrpTaskPriv {
 
 	/* List of assignments. */
 	GList            *assignments;
+
+	gfloat            cost;
+	gboolean          cost_cached;
 };
 
 
@@ -139,7 +144,6 @@ static void task_get_property          (GObject            *object,
 static void task_removed               (MrpObject          *object);
 static void task_assignment_removed_cb (MrpAssignment      *assignment,
 					MrpTask            *task);
-
 static void task_remove_assignments    (MrpTask            *task);
 static void task_remove_relations      (MrpTask            *task);
 
@@ -184,10 +188,13 @@ task_init (MrpTask *task)
 
 	priv->name = g_strdup ("");
 	priv->node = g_node_new (task);
-	priv->sorted_node = g_node_new (task);
 	priv->assignments = NULL;
 	priv->constraint.type = MRP_CONSTRAINT_ASAP;
+	priv->graph_node = g_new0 (MrpTaskGraphNode, 1);
 	priv->note = g_strdup ("");
+
+	priv->cost = 0.0;
+ 	priv->cost_cached = FALSE;
 }
 
 static void
@@ -414,7 +421,6 @@ task_finalize (GObject *object)
 	g_assert (priv->successors == NULL);
 	
 	g_node_destroy (priv->node);
-	g_node_destroy (priv->sorted_node);
 
 	g_free (priv);
 	task->priv = NULL;
@@ -488,6 +494,8 @@ task_set_property (GObject      *object,
 
 			g_object_notify (object, "work");
 			changed = TRUE;
+
+			mrp_task_invalidate_cost (task);
 		}
 		break;
 		
@@ -504,6 +512,8 @@ task_set_property (GObject      *object,
 
 			g_object_notify (object, "duration");
 			changed = TRUE;
+
+			mrp_task_invalidate_cost (task);
 		}
 		break;
 
@@ -752,8 +762,7 @@ task_remove_subtree_cb (GNode *node, gpointer data)
 	task_remove_assignments (task);
 
 	g_node_unlink (priv->node);
-	g_node_unlink (priv->sorted_node);
-
+	
 	mrp_object_removed (MRP_OBJECT (task));
 
 	g_object_unref (task);
@@ -788,6 +797,7 @@ imrp_task_remove_subtree (MrpTask *task)
 	g_object_unref (task);
 
 	if (parent) {
+		mrp_task_invalidate_cost (parent);
 		g_signal_emit (parent, signals[CHILD_REMOVED], 0);
 	}
 }
@@ -800,12 +810,6 @@ imrp_task_detach (MrpTask *task)
 	/* FIXME: Do some extra checking. */
 
 	g_node_unlink (task->priv->node);
-
-	/* Note: we don't unlink the sorted node, because when moving a task but
-	 * keeping it under the same parent, we don't need to rebuild the
-	 * tree. So when the caller wants that, it has to unlink the tree and
-	 * rebuild it.
-	 */
 }
 
 void
@@ -822,6 +826,14 @@ imrp_task_reattach (MrpTask  *task,
 
 	/* FIXME: Do some extra checking. */
 
+	if (parent->priv->type == MRP_TASK_TYPE_MILESTONE &&
+	    !parent->priv->node->children) {
+		g_object_set (parent,
+			      "type", MRP_TASK_TYPE_NORMAL,
+			      "sched", MRP_TASK_SCHED_FIXED_WORK,
+			      NULL);
+	}
+	
 	if (sibling == NULL) {
 		if (before) {
 			node = g_node_first_child (parent->priv->node);
@@ -935,9 +947,14 @@ imrp_task_insert_child (MrpTask *parent,
 		       position,
 		       child->priv->node);
 
-	g_node_prepend (parent->priv->sorted_node,
-			child->priv->sorted_node);
+	mrp_task_invalidate_cost (parent);
 
+	if (parent->priv->type == MRP_TASK_TYPE_MILESTONE) {
+		g_object_set (parent,
+			      "type", MRP_TASK_TYPE_NORMAL,
+			      NULL);
+	}
+	
 	g_signal_emit (parent, signals[CHILD_ADDED], 0);
 }
 
@@ -1261,6 +1278,25 @@ mrp_task_get_next_sibling (MrpTask *task)
 	return node ? node->data : NULL;
 }
 
+/**
+ * mrp_task_get_prev_sibling:
+ * @task: an #MrpTask
+ * 
+ * Fetches the previous sibling of @task.
+ * 
+ * Return value: the previous sibling of @task, or %NULL if there is none.
+ **/
+MrpTask *
+mrp_task_get_prev_sibling (MrpTask *task)
+{
+	GNode *node;
+
+	g_return_val_if_fail (MRP_IS_TASK (task), NULL);
+
+	node = g_node_prev_sibling (task->priv->node);
+	return node ? node->data : NULL;
+}
+
 /**
  * mrp_task_get_n_children:
  * @task: an #MrpTask
@@ -1584,15 +1620,22 @@ mrp_task_get_cost (MrpTask *task)
 	MrpResource *resource;
 	gfloat       total = 0;
 	gfloat       cost;
+	MrpTask     *child;
 
 	g_return_val_if_fail (MRP_IS_TASK (task), 0);
 
-	/* FIXME: why don't we cache this in priv->cost and recalculate in
-	 * reschedule? This is called a lot from the tree view...
-	 */
-
 	priv = task->priv;
 
+	if (priv->cost_cached) {
+		return priv->cost;
+	}
+
+	child = mrp_task_get_first_child (task);
+	while (child) {
+		total += mrp_task_get_cost (child);
+		child = mrp_task_get_next_sibling (child);
+	}
+
 	assignments = mrp_task_get_assignments (task);
 	for (l = assignments; l; l = l->next) {
 		resource = mrp_assignment_get_resource (l->data);
@@ -1601,9 +1644,24 @@ mrp_task_get_cost (MrpTask *task)
 		total += mrp_assignment_get_units (l->data) * priv->duration * cost / (3600.0 * 100);
 	}
 
+	priv->cost = total;
+	priv->cost_cached = TRUE;
+
 	return total;
 }
 
+void
+mrp_task_invalidate_cost (MrpTask *task)
+{
+	g_return_if_fail (MRP_IS_TASK (task));
+
+	task->priv->cost_cached = FALSE;
+
+	if (task->priv->node->parent) {
+		mrp_task_invalidate_cost (task->priv->node->parent->data);
+	}
+}
+
 /* Boxed types. */
 
 static MrpConstraint *
@@ -1737,12 +1795,12 @@ imrp_task_get_node (MrpTask *task)
 	return task->priv->node;
 }
 
-GNode *
-imrp_task_get_sorted_node (MrpTask *task)
+MrpTaskGraphNode *
+imrp_task_get_graph_node (MrpTask *task)
 {
 	g_return_val_if_fail (MRP_IS_TASK (task), NULL);
 	
-	return task->priv->sorted_node;
+	return task->priv->graph_node;
 }
 
 GList *
