@@ -3,6 +3,8 @@
  * Copyright (C) 2002 CodeFactory AB
  * Copyright (C) 2002 Richard Hult <richard@imendio.com>
  * Copyright (C) 2002 Mikael Hallendal <micke@imendio.com>
+ * Copyright (C) 2004 Imendio AB
+ * Copyright (C) 2004 Alvaro del Castillo <acs@barrapunto.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,7 +23,7 @@
  */
 
 #include <config.h>
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
 #include <glade/glade.h>
 #include <gtk/gtk.h>
 #include <libplanner/mrp-day.h>
@@ -46,7 +48,7 @@ enum {
 };
 
 typedef struct {
-	MgMainWindow  *main_window;
+	PlannerWindow *main_window;
 	MrpProject    *project;
 
 	GtkWidget     *dialog;
@@ -104,9 +106,9 @@ static void          cal_dialog_selection_changed_cb   (GtkTreeSelection *select
 static void          cal_dialog_apply_clicked_cb       (GtkWidget        *button,
 							DialogData       *data);
 static void          cal_dialog_update_calendar_widgets (DialogData      *data);
-static void          cal_dialog_month_changed_cb       (MgCalendar       *calendar,
+static void          cal_dialog_month_changed_cb       (PlannerCalendar       *calendar,
 							DialogData       *data);
-static void          cal_dialog_date_selected_cb       (MgCalendar       *calendar,
+static void          cal_dialog_date_selected_cb       (PlannerCalendar       *calendar,
 							DialogData       *data);
 static void          cal_dialog_option_menu_changed_cb (GtkWidget        *option_menu,
 							DialogData       *data);
@@ -126,6 +128,294 @@ static void          cal_dialog_project_day_changed_cb (MrpProject       *projec
 static void          cal_dialog_new_dialog_run         (DialogData       *data);
 
 
+typedef struct {
+	PlannerCmd        base;
+
+	MrpCalendar *calendar;
+	MrpDay      *day;
+	MrpDay      *old_day;
+	mrptime      time;
+} CalCmdDayType;
+
+static gboolean
+cal_cmd_day_type_do (PlannerCmd *cmd_base)
+{
+	CalCmdDayType *cmd = (CalCmdDayType*) cmd_base;
+
+	if (g_getenv ("PLANNER_DEBUG_UNDO_CAL")) {
+		g_message ("Changing calendar for day ...");
+	}
+
+	mrp_calendar_set_days (cmd->calendar, cmd->time, cmd->day, (mrptime) -1);
+
+	return TRUE;
+}
+
+static void
+cal_cmd_day_type_undo (PlannerCmd *cmd_base)
+{
+	CalCmdDayType *cmd = (CalCmdDayType*) cmd_base;
+
+	mrp_calendar_set_days (cmd->calendar, cmd->time, cmd->old_day, (mrptime) -1);
+}
+
+
+static void
+cal_cmd_day_type_free (PlannerCmd *cmd_base)
+{
+	CalCmdDayType *cmd = (CalCmdDayType*) cmd_base;
+
+	g_object_unref (cmd->calendar);
+	mrp_day_unref  (cmd->day);
+	mrp_day_unref  (cmd->old_day);
+}
+
+static PlannerCmd *
+planner_cal_cmd_day_type (PlannerWindow  *main_window,
+			  MrpCalendar    *calendar,
+			  MrpDay         *day,
+			  mrptime         time)
+{
+	PlannerCmd    *cmd_base;
+	CalCmdDayType *cmd;
+
+	cmd_base = planner_cmd_new (CalCmdDayType,
+				    _("Change day type"),
+				    cal_cmd_day_type_do,
+				    cal_cmd_day_type_undo,
+				    cal_cmd_day_type_free);
+
+	cmd = (CalCmdDayType*) cmd_base;
+
+	cmd->calendar = g_object_ref (calendar);
+	cmd->day = mrp_day_ref (day);
+	cmd->old_day = mrp_day_ref (mrp_calendar_get_day (calendar, time, FALSE));
+	cmd->time = time;
+			
+	planner_cmd_manager_insert_and_do (planner_window_get_cmd_manager 
+					   (main_window),
+					   cmd_base);
+	return cmd_base;
+}
+
+typedef struct {
+	PlannerCmd   base;
+
+	MrpProject  *project;
+	MrpCalendar *calendar;
+	MrpCalendar *parent;
+	/* Resources that use this calendar */
+	GList       *resources;
+	/* Calendar that inherits from this calendar */
+	GList       *children;
+} CalCmdRemove;
+
+static gboolean
+cal_cmd_remove_do (PlannerCmd *cmd_base)
+{
+	GList *all_resources, *r;
+
+	CalCmdRemove *cmd = (CalCmdRemove*) cmd_base;
+
+	if (g_getenv ("PLANNER_DEBUG_UNDO_CAL")) {
+		g_message ("Removing a calendar ...");
+	}
+
+	all_resources = mrp_project_get_resources (cmd->project);
+	for (r = all_resources; r; r = r->next) {
+		MrpResource *resource = r->data;
+		MrpCalendar *tmp_cal;
+		
+		tmp_cal = mrp_resource_get_calendar (resource);
+		if (tmp_cal == cmd->calendar) {			
+			cmd->resources = g_list_append (cmd->resources, 
+							g_object_ref (resource));
+		} 
+	}
+
+	cmd->children = g_list_copy (mrp_calendar_get_children (cmd->calendar));
+	if (cmd->children) {
+		g_list_foreach (cmd->children, (GFunc) g_object_ref, NULL);
+	}
+		
+	mrp_calendar_remove (cmd->calendar);
+
+	return TRUE;
+}
+
+/* Reassign resources and calendar childs */
+static void
+cal_cmd_remove_undo (PlannerCmd *cmd_base)
+{
+	GList        *r;
+	GList        *c;
+	CalCmdRemove *cmd = (CalCmdRemove*) cmd_base;
+
+	mrp_calendar_add (cmd->calendar, cmd->parent);
+
+	if (cmd->resources != NULL) {
+		for (r = cmd->resources; r; r = r->next) {
+			mrp_resource_set_calendar (r->data, cmd->calendar);
+		}
+		g_list_foreach (cmd->resources, (GFunc) g_object_unref, NULL);
+		g_list_free (cmd->resources);
+		cmd->resources = NULL;
+	}
+
+	if (cmd->children != NULL) {
+		for (c = cmd->children; c; c = c->next) {
+			mrp_calendar_reparent (cmd->calendar, c->data);
+		}
+		g_list_foreach (cmd->children, (GFunc) g_object_unref, NULL);
+		g_list_free (cmd->children);
+		cmd->children = NULL;
+	}
+}
+
+
+static void
+cal_cmd_remove_free (PlannerCmd *cmd_base)
+{
+	CalCmdRemove *cmd = (CalCmdRemove*) cmd_base;
+
+	g_object_unref (cmd->calendar);
+	g_object_unref (cmd->project);
+
+	if (cmd->resources != NULL) {
+		g_list_foreach (cmd->resources, (GFunc) g_object_unref, NULL);
+		g_list_free (cmd->resources);
+		cmd->resources = NULL;
+	}
+	if (cmd->children != NULL) {
+		g_list_foreach (cmd->children, (GFunc) g_object_unref, NULL);
+		g_list_free (cmd->children);
+		cmd->children = NULL;
+	}
+}
+
+static PlannerCmd *
+planner_cal_cmd_remove (PlannerWindow  *main_window,
+			MrpProject     *project,
+			MrpCalendar    *calendar)
+{
+	PlannerCmd   *cmd_base;
+	CalCmdRemove *cmd;
+
+	cmd_base = planner_cmd_new (CalCmdRemove,
+				    _("Remove calendar"),
+				    cal_cmd_remove_do,
+				    cal_cmd_remove_undo,
+				    cal_cmd_remove_free);
+
+	cmd = (CalCmdRemove*) cmd_base;
+
+	cmd->project = g_object_ref (project);
+	cmd->calendar = g_object_ref (calendar);
+	cmd->parent = g_object_ref (mrp_calendar_get_parent (cmd->calendar));
+
+	planner_cmd_manager_insert_and_do (planner_window_get_cmd_manager 
+					   (main_window),
+					   cmd_base);
+	return cmd_base;
+}
+
+typedef struct {
+	PlannerCmd   base;
+
+	gchar       *name;
+	MrpProject  *project;
+	MrpCalendar *calendar;
+	MrpCalendar *parent;
+	MrpCalendar *copy;
+} CalCmdAdd;
+
+static gboolean
+cal_cmd_add_do (PlannerCmd *cmd_base)
+{
+	CalCmdAdd *cmd = (CalCmdAdd*) cmd_base;
+
+	if (g_getenv ("PLANNER_DEBUG_UNDO_CAL")) {
+		g_message ("Adding a new calendar ...");
+	}
+
+	if (cmd->calendar == NULL) {
+		if (cmd->parent != NULL && cmd->copy == NULL) {
+			cmd->calendar = mrp_calendar_derive (cmd->name, cmd->parent);
+			g_object_unref (cmd->parent);
+			cmd->parent = NULL;
+		} 
+		else if (cmd->parent == NULL && cmd->copy != NULL) {
+			cmd->calendar = mrp_calendar_copy (cmd->name, cmd->copy);
+			g_object_unref (cmd->copy);
+			cmd->copy = NULL;
+		} 
+		else if (cmd->parent == NULL && cmd->copy == NULL) {
+			cmd->calendar = mrp_calendar_new (cmd->name, cmd->project);
+			cmd->parent = g_object_ref (mrp_calendar_get_parent (cmd->calendar));
+		} else {
+			g_warning ("Incorrect use adding new calendar");
+		}
+		cmd->parent = g_object_ref (mrp_calendar_get_parent (cmd->calendar));
+	} else {
+		mrp_calendar_add (cmd->calendar, cmd->parent);
+	}
+
+	return TRUE;
+}
+
+static void
+cal_cmd_add_undo (PlannerCmd *cmd_base)
+{
+	CalCmdAdd *cmd = (CalCmdAdd*) cmd_base;
+
+	mrp_calendar_remove (cmd->calendar);
+}
+
+
+static void
+cal_cmd_add_free (PlannerCmd *cmd_base)
+{
+	CalCmdAdd *cmd = (CalCmdAdd*) cmd_base;
+
+	g_object_unref (cmd->calendar);
+	g_object_unref (cmd->parent);
+	g_object_unref (cmd->project);
+
+	g_free (cmd->name);
+}
+
+static PlannerCmd *
+planner_cal_cmd_add (PlannerWindow  *main_window,
+		     const gchar    *name,
+		     MrpCalendar    *parent,
+		     MrpCalendar    *copy)
+{
+	PlannerCmd *cmd_base;
+	CalCmdAdd  *cmd;
+
+	cmd_base = planner_cmd_new (CalCmdAdd,
+				    _("Add new calendar"),
+				    cal_cmd_add_do,
+				    cal_cmd_add_undo,
+				    cal_cmd_add_free);
+
+	cmd = (CalCmdAdd*) cmd_base;
+
+	cmd->project = g_object_ref (planner_window_get_project (main_window));
+	cmd->name = g_strdup (name);
+	if (parent != NULL) {
+		cmd->parent = g_object_ref (parent);
+	}
+	if (copy != NULL) {
+		cmd->copy = g_object_ref (copy);
+	}
+			
+	planner_cmd_manager_insert_and_do (planner_window_get_cmd_manager 
+					   (main_window),
+					   cmd_base);
+	return cmd_base;
+}
+
 static void
 cal_dialog_setup_option_menu (DialogData *data)
 {
@@ -216,7 +506,8 @@ cal_dialog_response_cb (GtkWidget  *dialog,
 	
 	switch (response) {
 	case RESPONSE_REMOVE:
-		mrp_calendar_remove (calendar);
+		/* mrp_calendar_remove (calendar); */
+		planner_cal_cmd_remove (data->main_window, data->project, calendar);
 		break;
 
 	case RESPONSE_ADD:
@@ -241,7 +532,7 @@ cal_dialog_response_cb (GtkWidget  *dialog,
 		window = planner_default_week_dialog_new (data->main_window, calendar);
 
 		/* FIXME: Not very nice to have it modal, but I can't think of a
-		 *  better way right now.
+		 * better way right now.
 		 */
 		gtk_window_set_transient_for (GTK_WINDOW (window),
 					      GTK_WINDOW (data->dialog));
@@ -288,7 +579,7 @@ cal_dialog_parent_destroy_cb (GtkWidget *window, GtkWidget *dialog)
 }
 	
 GtkWidget *
-planner_calendar_dialog_new (MgMainWindow *window)
+planner_calendar_dialog_new (PlannerWindow *window)
 {
 	DialogData       *data;
 	GladeXML         *glade;
@@ -297,7 +588,7 @@ planner_calendar_dialog_new (MgMainWindow *window)
 	GtkTreeSelection *selection;
 	gint              i;
 	
-	g_return_val_if_fail (MG_IS_MAIN_WINDOW (window), NULL);
+	g_return_val_if_fail (PLANNER_IS_MAIN_WINDOW (window), NULL);
 	
 	glade = glade_xml_new (GLADEDIR "/calendar-dialog.glade",
 			       "calendar_dialog",
@@ -311,7 +602,7 @@ planner_calendar_dialog_new (MgMainWindow *window)
 	
 	data = g_new0 (DialogData, 1);
 
-	data->project = planner_main_window_get_project (window);
+	data->project = planner_window_get_project (window);
 	data->main_window = window;
 	data->dialog = dialog;
 
@@ -325,11 +616,11 @@ planner_calendar_dialog_new (MgMainWindow *window)
 	data->calendar = planner_calendar_new ();
 	gtk_widget_show (data->calendar);
 
-	planner_calendar_display_options (MG_CALENDAR (data->calendar),
-				     MG_CALENDAR_SHOW_HEADING |
-				     MG_CALENDAR_SHOW_DAY_NAMES |
-				     MG_CALENDAR_SHOW_WEEK_NUMBERS |
-				     MG_CALENDAR_WEEK_START_MONDAY);
+	planner_calendar_display_options (PLANNER_CALENDAR (data->calendar),
+					  PLANNER_CALENDAR_SHOW_HEADING |
+					  PLANNER_CALENDAR_SHOW_DAY_NAMES |
+					  PLANNER_CALENDAR_SHOW_WEEK_NUMBERS |
+					  PLANNER_CALENDAR_WEEK_START_MONDAY);
 	g_signal_connect (data->calendar,
 			  "month_changed",
 			  G_CALLBACK (cal_dialog_month_changed_cb),
@@ -545,9 +836,10 @@ cal_dialog_apply_clicked_cb (GtkWidget  *button,
 		return;
 	}
 	
-	planner_calendar_get_date (MG_CALENDAR (data->calendar), &y, &m, &d);
+	planner_calendar_get_date (PLANNER_CALENDAR (data->calendar), &y, &m, &d);
 	t = mrp_time_compose (y, m + 1, d, 0, 0, 0);
-	mrp_calendar_set_days (calendar, t, day, -1);
+	/* mrp_calendar_set_days (calendar, t, day, (mrptime) -1); */
+	planner_cal_cmd_day_type (data->main_window, calendar, day, t);
 }
 
 static void
@@ -577,7 +869,7 @@ static void
 cal_dialog_update_calendar_widgets (DialogData *data)
 {
 	MrpCalendar *calendar;
-	MgCalendar  *calendar_widget;
+	PlannerCalendar  *calendar_widget;
 	guint        y, m, d;
 	mrptime      t;
 	MrpDay      *day;
@@ -590,7 +882,7 @@ cal_dialog_update_calendar_widgets (DialogData *data)
 		return;
 	}
 
-	calendar_widget = MG_CALENDAR (data->calendar);
+	calendar_widget = PLANNER_CALENDAR (data->calendar);
 
 	gtk_widget_set_sensitive (data->calendar, TRUE);
 	gtk_widget_set_sensitive (data->default_week_button, TRUE);
@@ -605,19 +897,19 @@ cal_dialog_update_calendar_widgets (DialogData *data)
 
 		if (day == mrp_day_get_nonwork ()) {
 			planner_calendar_mark_day (calendar_widget,
-					      d,
-					      MG_CALENDAR_MARK_SHADE);
+						   d,
+						   PLANNER_CALENDAR_MARK_SHADE);
 		}
 		else if (day == mrp_day_get_work ()) {
 			planner_calendar_mark_day (calendar_widget,
-					      d,
-					      MG_CALENDAR_MARK_NONE);
+						   d,
+						   PLANNER_CALENDAR_MARK_NONE);
 		}
 	}
 }
 
 static void
-cal_dialog_month_changed_cb (MgCalendar *calendar_widget,
+cal_dialog_month_changed_cb (PlannerCalendar *calendar_widget,
 			     DialogData *data)
 {
 	cal_dialog_update_calendar_widgets (data);
@@ -625,7 +917,7 @@ cal_dialog_month_changed_cb (MgCalendar *calendar_widget,
 }
 
 static void
-cal_dialog_date_selected_cb (MgCalendar *calendar_widget,
+cal_dialog_date_selected_cb (PlannerCalendar *calendar_widget,
 			     DialogData *data)
 {
 	cal_dialog_update_day_widgets (data);
@@ -664,7 +956,7 @@ cal_dialog_update_day_widgets (DialogData *data)
 	MrpDay      *day;
 	GList       *ivals, *l;
 	MrpCalendar *calendar, *root;
-	MgCalendar  *calendar_widget;
+	PlannerCalendar  *calendar_widget;
 	guint        y, m, d;
 	mrptime      t;
 	gint         i;
@@ -708,7 +1000,7 @@ cal_dialog_update_day_widgets (DialogData *data)
 		gtk_widget_set_sensitive (data->base_radiobutton, TRUE);
 	}
 
-	calendar_widget = MG_CALENDAR (data->calendar);
+	calendar_widget = PLANNER_CALENDAR (data->calendar);
 	
 	planner_calendar_get_date (calendar_widget, &y, &m, &d);
 	t = mrp_time_compose (y, m + 1, d, 0, 0, 0);
@@ -949,7 +1241,6 @@ cal_dialog_new_dialog_run (DialogData *data)
 	GladeXML         *glade;
 	GtkWidget        *dialog;
 	MrpCalendar      *parent;
-	MrpCalendar      *calendar;
 	GtkTreeSelection *selection;
 	GtkWidget        *entry;
 	GtkWidget        *tree_view;
@@ -1004,13 +1295,17 @@ cal_dialog_new_dialog_run (DialogData *data)
 		parent = cal_dialog_get_selected_calendar (GTK_TREE_VIEW (tree_view));
 
 		if (parent && gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->new_copy_radiobutton))) {
-			calendar = mrp_calendar_copy (name, parent);
+			/* calendar = mrp_calendar_copy (name, parent); */
+			planner_cal_cmd_add (data->main_window, name, NULL, parent);
 		}
 		else if (parent && gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->new_derive_radiobutton))) {
-			calendar = mrp_calendar_derive (name, parent);
+			/* calendar = mrp_calendar_derive (name, parent); */
+			planner_cal_cmd_add (data->main_window, name, parent, NULL);
 		}
 		else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->new_empty_radiobutton))) {
-			calendar = mrp_calendar_new (name, data->project);
+			/* calendar = mrp_calendar_new (name, data->project); */
+			planner_cal_cmd_add (data->main_window, name, NULL, NULL);
+			   
 		}
 	}
 
