@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2004      Imendio HB
+ * Copyright (C) 2004      Imendio AB
  * Copyright (C) 2001-2002 CodeFactory AB
  * Copyright (C) 2001-2002 Richard Hult <richard@imendio.com>
  * Copyright (C) 2001-2002 Mikael Hallendal <micke@imendio.com>
@@ -28,9 +28,10 @@
 #include <math.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
 #include <libplanner/mrp-task.h>
 #include "planner-marshal.h"
+#include "planner-conf.h"
 #include "planner-gantt-chart.h"
 #include "planner-gantt-header.h"
 #include "planner-gantt-background.h"
@@ -52,7 +53,7 @@
 /* Font width factor. */
 static gdouble f = 1.0;
 
-#define CRITICAL_PATH_KEY "/apps/planner/views/gantt_view/highlight_critical_path"
+#define CRITICAL_PATH_KEY "/views/gantt_view/highlight_critical_path"
 
 
 typedef struct _TreeNode TreeNode; 
@@ -191,6 +192,9 @@ static void        gantt_chart_set_zoom                 (PlannerGanttChart  *cha
 static gint        gantt_chart_get_width                (PlannerGanttChart  *chart);
 static PlannerGanttRow *gantt_chart_get_row_from_task   (PlannerGanttChart  *chart,
 							 MrpTask            *task);
+static void        gantt_chart_header_date_hint_changed_cb (GtkWidget         *header,
+							    const gchar       *hint,
+							    PlannerGanttChart *chart);
 static TreeNode *  gantt_chart_tree_node_new            (void);
 static void        gantt_chart_tree_node_insert_path    (TreeNode           *node,
 							 GtkTreePath        *path,
@@ -320,7 +324,6 @@ static void
 gantt_chart_init (PlannerGanttChart *chart)
 {
 	PlannerGanttChartPriv *priv;
-	GConfClient           *gconf_client;
 	
 	gtk_widget_set_redraw_on_allocate (GTK_WIDGET (chart), FALSE);
 
@@ -341,6 +344,11 @@ gantt_chart_init (PlannerGanttChart *chart)
 				     "scale", SCALE (priv->zoom),
 				     "zoom", priv->zoom,
 				     NULL);
+
+	g_signal_connect (priv->header,
+			  "date_hint_changed",
+			  G_CALLBACK (gantt_chart_header_date_hint_changed_cb),
+			  chart);
 	
 	gtk_box_pack_start (GTK_BOX (chart),
 			    GTK_WIDGET (priv->header),
@@ -374,9 +382,8 @@ gantt_chart_init (PlannerGanttChart *chart)
 
 	priv->relation_hash = g_hash_table_new (NULL, NULL);
 
-	gconf_client = planner_application_get_gconf_client ();
-	priv->highlight_critical = gconf_client_get_bool (
-		gconf_client, CRITICAL_PATH_KEY, NULL);
+	priv->highlight_critical = planner_conf_get_bool (CRITICAL_PATH_KEY,
+							  NULL);
 }
 
 static void
@@ -1488,7 +1495,7 @@ planner_gantt_chart_set_model (PlannerGanttChart *chart,
 		g_object_set (priv->background,
 			      "project-start", priv->project_start,
 			      NULL);
-	
+
 		priv->last_time = mrp_task_get_finish (root);
 		
 		/* Force a reflow initially to avoid visible reflow on
@@ -1682,6 +1689,14 @@ gantt_chart_get_row_from_task (PlannerGanttChart *chart,
 	return PLANNER_GANTT_ROW (node->item);
 }
 
+static void
+gantt_chart_header_date_hint_changed_cb (GtkWidget         *header,
+					 const gchar       *hint,
+					 PlannerGanttChart *chart)
+{
+	planner_gantt_chart_status_updated (chart, hint);
+}
+
 static void
 scale_func (TreeNode *node, gpointer data)
 {
@@ -1879,7 +1894,6 @@ planner_gantt_chart_set_highlight_critical_tasks (PlannerGanttChart *chart,
 						  gboolean           state)
 {
 	PlannerGanttChartPriv *priv;
-	GConfClient           *gconf_client;
 	
 	g_return_if_fail (PLANNER_IS_GANTT_CHART (chart));
 
@@ -1893,11 +1907,7 @@ planner_gantt_chart_set_highlight_critical_tasks (PlannerGanttChart *chart,
 	
 	gtk_widget_queue_draw (GTK_WIDGET (priv->canvas));
 
-	gconf_client = planner_application_get_gconf_client ();
-	gconf_client_set_bool (gconf_client,
-			       CRITICAL_PATH_KEY,
-			       state,
-			       NULL);
+	planner_conf_set_bool (CRITICAL_PATH_KEY, state, NULL);
 }
 
 gboolean
