@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2004 Imendio HB
+ * Copyright (C) 2004 Imendio AB
  * Copyright (C) 2003 CodeFactory AB
  * Copyright (C) 2003 Richard Hult <richard@imendio.com>
  * Copyright (C) 2003 Mikael Hallendal <micke@imendio.com>
@@ -27,7 +27,7 @@
 #include <libplanner/mrp-project.h>
 #include <libplanner/mrp-task.h>
 #include <libplanner/mrp-resource.h>
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
 #include <libgnomeprint/gnome-print.h>
 #include "planner-print-job.h"
 #include "planner-format.h"
@@ -151,7 +151,8 @@ typedef enum {
 
 	RESOURCES,
 
-	SHADE
+	SHADE,
+	TIMELINE
 } ElementType;
 
 typedef struct {
@@ -257,7 +258,7 @@ print_table_tasks (PlannerGanttPrintData *data,
 		
 		x = data->work_x1 + data->job->x_pad;
 		
-		str = planner_format_duration (work, 8);
+		str = planner_format_duration (data->project, work);
 		planner_print_job_show_clipped (data->job,
 						x, y,
 						str,
@@ -584,12 +585,13 @@ gantt_print_get_allocated_resources_string (PlannerGanttPrintData  *data,
 void
 planner_gantt_print_do (PlannerGanttPrintData *data)
 {
-	GList      *relations;
-	GList      *l;
+	GList       *relations;
+	GList       *l;
 	gdouble      x1, x2;
 	gdouble      y1, y2;
 	gint         num_tasks;
 	mrptime      t1, t2, t0;
+	mrptime      current_time;
 	mrptime      start, finish;
 	mrptime      complete;
 	gboolean     is_summary;
@@ -613,6 +615,8 @@ planner_gantt_print_do (PlannerGanttPrintData *data)
 	calendar = mrp_project_get_calendar (data->project);
 	
 	num_tasks = g_list_length (data->tasks);
+	
+	current_time = mrp_time_current_time();
 
 	/* Go through all tasks in a first pass and layout them so that we know
 	 * where to draw relation arrows.
@@ -700,6 +704,7 @@ planner_gantt_print_do (PlannerGanttPrintData *data)
 					element->x1 = x0 + (start - t1) / data->f;
 					element->x2 = data->job->width;
 					element->x_complete = x0 + (complete - t1) / data->f;
+					element->x_complete = MIN (element->x_complete, element->x2);
 				}
 				else if (start < t1 && finish >= t1 && finish <= t2) {
 					/* Right */
@@ -717,6 +722,7 @@ planner_gantt_print_do (PlannerGanttPrintData *data)
 					element->x1 = x0;
 					element->x2 = x0 + (finish - t1) / data->f;
 					element->x_complete = x0 + (complete - t1) / data->f;
+					element->x_complete = MIN (element->x_complete, element->x2);
 				}
 				else if (start >= t1 && finish <= t2) {
 					/* Whole */
@@ -734,6 +740,7 @@ planner_gantt_print_do (PlannerGanttPrintData *data)
 					element->x1 = x0 + (start - t1) / data->f;
 					element->x2 = x0 + (finish - t1) / data->f;
 					element->x_complete = x0 + (complete - t1) / data->f;
+					element->x_complete = MIN (element->x_complete, element->x2);
 				}
 				else if (start < t1 && finish > t2) {
 					/* Middle */
@@ -750,7 +757,11 @@ planner_gantt_print_do (PlannerGanttPrintData *data)
 					
 					element->x1 = x0;
 					element->x2 = data->job->width;
-					element->x_complete = data->job->width;
+
+					if (complete > t1 && complete <= t2) {
+						element->x_complete = x0 + (complete - t1) / data->f;
+						element->x_complete = MIN (element->x_complete, element->x2);
+					}
 				} else {
 					d(g_print ("nothing"));
 					g_free (element);
@@ -1059,6 +1070,18 @@ planner_gantt_print_do (PlannerGanttPrintData *data)
 				}
 			}
 
+			/* Print the current time. */
+			if (current_time >= t0 && current_time <= t2) {
+				element = g_new0 (Element, 1);
+				element->type = TIMELINE;
+				element->y1 = data->header_height + data->row_height / 4;
+				element->y2 = data->job->height;
+				element->x1 = x0 + (current_time - t0) / data->f; 
+				element->x2 = element->x1;
+				page = GET_PAGE (data, row, col);
+				page->background_elements = g_list_prepend (page->background_elements, element);
+			}
+			
 			t0 = t2;
 		}
 	}
@@ -1103,9 +1126,26 @@ planner_gantt_print_do (PlannerGanttPrintData *data)
 
 			page = GET_PAGE (data, row, col);
 			for (l = page->background_elements; l; l = l->next) {
+				gdouble dashes[] = { 4, 4 };
+				
 				element = l->data;
-
+				
 				switch (element->type) {
+				case TIMELINE:
+                                        gnome_print_newpath (data->job->pc);
+					gnome_print_setrgbcolor (data->job->pc, 150/255.0, 150/255.0, 249/255.0);
+                                        gnome_print_setlinewidth (data->job->pc, 1);
+					
+					gnome_print_setdash (data->job->pc, 2, dashes, 0);
+
+					gnome_print_setlinewidth (data->job->pc, 1);
+                                        planner_print_job_moveto (data->job, element->x1, element->y1);
+                                        planner_print_job_lineto (data->job,element->x1, element->y2);
+                                        gnome_print_stroke (data->job->pc);
+
+					gnome_print_setdash (data->job->pc, 0, NULL, 0);
+
+                                        break;
 				case SHADE:
 					gnome_print_newpath (data->job->pc);
 					planner_print_job_moveto (data->job, element->x1, element->y1);
@@ -1129,7 +1169,7 @@ planner_gantt_print_do (PlannerGanttPrintData *data)
 			}
 
 			gnome_print_setrgbcolor (data->job->pc, 0, 0, 0);
-
+				
 			if (row == 0) {
 				print_time_header (data, x1, x2, t1, t2);
 				if (col == 0) {
