@@ -36,6 +36,38 @@
 
 /*
 
+SS:
+
+  1:      A 
+  	 +--XXXXXX
+       B | 
+	 +---XXXXX
+          C
+	  
+FF:
+
+  1:               A 
+  	   XXXXXX----+
+	             | B
+	     XXXXX---+
+		   C
+		   
+SF:
+
+  1:                       A (we have a maximum value for A)
+  	   		 +------XXXXXX
+	               B |
+		     XXXXX
+
+
+  2:	          A (max value)  
+     	         +-XXXXXX
+	       B |        C
+	 	 +---------------+
+	  		         |D
+	 		   XXXXX-+
+          		        E (max value)
+
 FS:
 
   1:               A (we have a minimum value for A)
@@ -53,34 +85,32 @@ FS:
           E (min value)
 
 
-TODO: Implement other types, remove the arguments from the geometry-changed signal.
-	 
 */
 
 
 typedef struct {
 	gdouble x;
 	gdouble y;
-} MgPoint;
+} PlannerPoint;
 
 typedef enum {
-	MG_ARROW_UP,
-	MG_ARROW_DOWN,
-	MG_ARROW_RIGHT,
-	MG_ARROW_LEFT
-} MgArrowDir;
+	PLANNER_ARROW_UP,
+	PLANNER_ARROW_DOWN,
+	PLANNER_ARROW_RIGHT,
+	PLANNER_ARROW_LEFT
+} PlannerArrowDir;
 	
-struct _MgRelationArrowPriv {
-	MgGanttRow      *successor;
-	MgGanttRow      *predecessor;
+struct _PlannerRelationArrowPriv {
+	PlannerGanttRow      *successor;
+	PlannerGanttRow      *predecessor;
 	MrpRelationType  type;
 
 	gboolean         successor_visible;
 	gboolean         predecessor_visible;
 
 	guint            num_points;
-	MgPoint          points[6];
-	MgArrowDir       arrow_dir;
+	PlannerPoint          points[6];
+	PlannerArrowDir       arrow_dir;
 		
 	gdouble          x1;
 	gdouble          y1;
@@ -91,8 +121,8 @@ struct _MgRelationArrowPriv {
 };
 
 
-static void    relation_arrow_init         (MgRelationArrow       *mra);
-static void    relation_arrow_class_init   (MgRelationArrowClass  *klass);
+static void    relation_arrow_init         (PlannerRelationArrow       *mra);
+static void    relation_arrow_class_init   (PlannerRelationArrowClass  *klass);
 static void    relation_arrow_finalize     (GObject               *object);
 static void    relation_arrow_set_property (GObject               *object,
 					    guint                  prop_id,
@@ -133,19 +163,19 @@ planner_relation_arrow_get_type (void)
 
 	if (!type) {
 		static const GTypeInfo info = {
-			sizeof (MgRelationArrowClass),
+			sizeof (PlannerRelationArrowClass),
 			NULL,		/* base_init */
 			NULL,		/* base_finalize */
 			(GClassInitFunc) relation_arrow_class_init,
 			NULL,		/* class_finalize */
 			NULL,		/* class_data */
-			sizeof (MgRelationArrow),
+			sizeof (PlannerRelationArrow),
 			0,              /* n_preallocs */
 			(GInstanceInitFunc) relation_arrow_init
 		};
 
 		type = g_type_register_static (GNOME_TYPE_CANVAS_ITEM,
-					       "MgRelationArrow",
+					       "PlannerRelationArrow",
 					       &info,
 					       0);
 	}
@@ -154,7 +184,7 @@ planner_relation_arrow_get_type (void)
 }
 
 static void
-relation_arrow_class_init (MgRelationArrowClass *klass)
+relation_arrow_class_init (PlannerRelationArrowClass *klass)
 {
 	GnomeCanvasItemClass *item_class;
 	GObjectClass         *o_class;
@@ -184,25 +214,25 @@ relation_arrow_class_init (MgRelationArrowClass *klass)
 }
 
 static void
-relation_arrow_init (MgRelationArrow *item)
+relation_arrow_init (PlannerRelationArrow *item)
 {
-	MgRelationArrowPriv *priv;
+	PlannerRelationArrowPriv *priv;
 
-	priv = g_new0 (MgRelationArrowPriv, 1);
+	priv = g_new0 (PlannerRelationArrowPriv, 1);
 	item->priv = priv;
 	
 	priv->successor_visible = TRUE;
 	priv->predecessor_visible = TRUE;
 
 	priv->type = MRP_RELATION_FS;
-	priv->arrow_dir = MG_ARROW_DOWN;
+	priv->arrow_dir = PLANNER_ARROW_DOWN;
 }
 
 static void
 relation_arrow_finalize (GObject *object)
 {
-	MgRelationArrow     *arrow = MG_RELATION_ARROW (object);
-	MgRelationArrowPriv *priv;
+	PlannerRelationArrow     *arrow = PLANNER_RELATION_ARROW (object);
+	PlannerRelationArrowPriv *priv;
 
 	priv = arrow->priv;
 	
@@ -230,10 +260,10 @@ relation_arrow_set_property (GObject      *object,
 			     const GValue *value,
 			     GParamSpec   *pspec)
 {
-	MgRelationArrow *arrow;
+	PlannerRelationArrow *arrow;
+
+	arrow = PLANNER_RELATION_ARROW (object);
 
-	arrow = MG_RELATION_ARROW (object);
-	
 	switch (prop_id) {
 	case PROP_TYPE:
 		arrow->priv->type = g_value_get_enum (value);
@@ -245,27 +275,130 @@ relation_arrow_set_property (GObject      *object,
 }
 
 static void
-relation_arrow_update_line_segments (MgRelationArrow *arrow)
+relation_arrow_update_line_segments (PlannerRelationArrow *arrow)
 {
-	MgRelationArrowPriv *priv;
-	gdouble              px1, py1, px2, py2;
-	gdouble              sx1, sy1, sx2, sy2;
-	gdouble              y;
+	PlannerRelationArrowPriv *priv;
+	gdouble                   px1, py1, px2, py2;
+	gdouble                   sx1, sy1, sx2, sy2;
+	gdouble                   y;
+	MrpRelationType           type;
 
 	priv = arrow->priv;
+	type = priv->type;
 
 	planner_gantt_row_get_geometry (priv->predecessor,
-				   &px1,
-				   &py1,
-				   &px2,
-				   &py2);
+					&px1,
+					&py1,
+					&px2,
+					&py2);
 	
 	planner_gantt_row_get_geometry (priv->successor,
-				   &sx1,
-				   &sy1,
-				   &sx2,
-				   &sy2);
+					&sx1,
+					&sy1,
+					&sx2,
+					&sy2);
+	if (type == MRP_RELATION_SS) {
+		priv->num_points = 4;
+		priv->arrow_dir = PLANNER_ARROW_RIGHT;
+		
+		/* LHS of pred */
+		priv->points[0].x = px1;
+		priv->points[0].y = py1 + (py2 - py1) / 2;
 
+		/* Next two, a bit left of the most left */
+		if (sx1 < px1) {
+			priv->points[1].x = sx1 - MIN_SPACING - ARROW_SIZE;
+			priv->points[1].y = py1 + (py2 - py1) / 2;
+			priv->points[2].x = sx1 - MIN_SPACING - ARROW_SIZE;
+			priv->points[2].y = sy1 + (sy2 - sy1) / 2;
+		} else {
+			priv->points[1].x = px1 - MIN_SPACING - ARROW_SIZE;
+			priv->points[1].y = py1 + (py2 - py1) / 2;
+			priv->points[2].x = px1 - MIN_SPACING - ARROW_SIZE;
+			priv->points[2].y = sy1 + (sy2 - sy1) / 2;
+		}
+		
+		priv->points[3].x = sx1;
+		priv->points[3].y = sy1 + (sy2 - sy1) / 2;
+		
+	}
+	else if (type == MRP_RELATION_FF) {
+		
+		priv->num_points = 4;
+		priv->arrow_dir = PLANNER_ARROW_LEFT;
+
+		/* LHS of pred */
+		priv->points[0].x = px2;
+		priv->points[0].y = py1 + (py2 - py1) / 2;
+
+		/* Next two, a bit left of the most left */
+		if (sx2 > px2) {
+			priv->points[1].x = sx2 + MIN_SPACING + ARROW_SIZE;
+			priv->points[1].y = py1 + (py2 - py1) / 2;
+			priv->points[2].x = sx2 + MIN_SPACING + ARROW_SIZE;
+			priv->points[2].y = sy1 + (sy2 - sy1) / 2;
+		} else {
+			priv->points[1].x = px2 + MIN_SPACING + ARROW_SIZE;
+			priv->points[1].y = py1 + (py2 - py1) / 2;
+			priv->points[2].x = px2 + MIN_SPACING + ARROW_SIZE;
+			priv->points[2].y = sy1 + (sy2 - sy1) / 2;
+		}
+		
+		priv->points[3].x = sx2;
+		priv->points[3].y = sy1 + (sy2 - sy1) / 2;
+	}
+	else if (type == MRP_RELATION_SF) {
+		/* Two cases for SF, as shown at the top of this file. */
+		if (px1 >= sx2) {
+			priv->num_points = 3;
+	
+			priv->points[0].x = px1;
+			priv->points[0].y = py1 + (py2 - py1) / 2;
+
+			priv->points[1].x = MIN (px1 - MIN_SPACING, sx2);
+			priv->points[1].y = py1 + (py2 - py1) / 2;
+	
+			priv->points[2].x = MIN (px1 - MIN_SPACING, sx2);
+			if (sy1 > py1) {
+				priv->points[2].y = sy1;
+				priv->arrow_dir = PLANNER_ARROW_DOWN;
+			} else {
+				priv->points[2].y = sy2;
+				priv->arrow_dir = PLANNER_ARROW_UP;
+			}
+	
+			
+		} else {
+			priv->num_points = 6;
+			priv->arrow_dir = PLANNER_ARROW_LEFT;
+	
+			priv->points[0].x = px1;
+			priv->points[0].y = py1 + (py2 - py1) / 2;
+	
+			priv->points[1].x = px1 - MIN_SPACING;
+			priv->points[1].y = py1 + (py2 - py1) / 2;
+	
+	 		if (sy1 > py1) {
+				y = py2 + (py2 - py1) / 2 - 1;
+			} else {
+				y = py1 - (py2 - py1) / 2 + 2;
+			}
+			
+			priv->points[2].x = px1 - MIN_SPACING;
+			priv->points[2].y = y;
+	
+			priv->points[3].x = sx2 + ARROW_SIZE + MIN_SPACING;
+			priv->points[3].y = y;
+			
+			priv->points[4].x = sx2 + ARROW_SIZE + MIN_SPACING;
+			priv->points[4].y = sy1 + (sy2 - sy1) / 2;
+	
+			priv->points[5].x = sx2;
+			priv->points[5].y = sy1 + (sy2 - sy1) / 2;
+		}
+	
+	
+	} else {
 	/* Two cases for FS, as shown at the top of this file. */
 	if (px2 <= sx1) {
 		priv->num_points = 3;
@@ -279,14 +412,16 @@ relation_arrow_update_line_segments (MgRelationArrow *arrow)
 		priv->points[2].x = MAX (px2 + MIN_SPACING, sx1);
 		if (sy1 > py1) {
 			priv->points[2].y = sy1;
-			priv->arrow_dir = MG_ARROW_DOWN;
+			priv->arrow_dir = PLANNER_ARROW_DOWN;
 		} else {
 			priv->points[2].y = sy2;
-			priv->arrow_dir = MG_ARROW_UP;
+			priv->arrow_dir = PLANNER_ARROW_UP;
 		}
+	
+			
 	} else {
 		priv->num_points = 6;
-		priv->arrow_dir = MG_ARROW_RIGHT;
+		priv->arrow_dir = PLANNER_ARROW_RIGHT;
 
 		priv->points[0].x = px2;
 		priv->points[0].y = py1 + (py2 - py1) / 2;
@@ -312,25 +447,26 @@ relation_arrow_update_line_segments (MgRelationArrow *arrow)
 		priv->points[5].x = sx1;
 		priv->points[5].y = sy1 + (sy2 - sy1) / 2;
 	}
+	}
 
 	gnome_canvas_item_request_update (GNOME_CANVAS_ITEM (arrow));
 }
 
 static void
-relation_arrow_geometry_changed (MgGanttRow      *row,
-				 gdouble          x1,
-				 gdouble          y1,
-				 gdouble          x2,
-				 gdouble          y2,
-				 MgRelationArrow *arrow)
+relation_arrow_geometry_changed (PlannerGanttRow      *row,
+				 gdouble               x1,
+				 gdouble               y1,
+				 gdouble               x2,
+				 gdouble               y2,
+				 PlannerRelationArrow *arrow)
 {
 	relation_arrow_update_line_segments (arrow);
 }
 
 static void
-relation_arrow_successor_visibility_changed (MgGanttRow      *row,
-					     gboolean         visible,
-					     MgRelationArrow *arrow)
+relation_arrow_successor_visibility_changed (PlannerGanttRow      *row,
+					     gboolean              visible,
+					     PlannerRelationArrow *arrow)
 {
 	arrow->priv->successor_visible = visible;
 	
@@ -342,9 +478,9 @@ relation_arrow_successor_visibility_changed (MgGanttRow      *row,
 }
 
 static void
-relation_arrow_predecessor_visibility_changed (MgGanttRow      *row,
-					       gboolean         visible,
-					       MgRelationArrow *arrow)
+relation_arrow_predecessor_visibility_changed (PlannerGanttRow      *row,
+					       gboolean              visible,
+					       PlannerRelationArrow *arrow)
 {
 	arrow->priv->predecessor_visible = visible;
 
@@ -356,16 +492,16 @@ relation_arrow_predecessor_visibility_changed (MgGanttRow      *row,
 }
 
 void
-planner_relation_arrow_set_predecessor (MgRelationArrow *arrow,
-				   MgGanttRow      *predecessor)
+planner_relation_arrow_set_predecessor (PlannerRelationArrow *arrow,
+					PlannerGanttRow      *predecessor)
 {
-	MgRelationArrowPriv *priv; 
+	PlannerRelationArrowPriv *priv; 
 	
-	g_return_if_fail (MG_IS_RELATION_ARROW (arrow));
-	g_return_if_fail (MG_IS_GANTT_ROW (predecessor));
+	g_return_if_fail (PLANNER_IS_RELATION_ARROW (arrow));
+	g_return_if_fail (PLANNER_IS_GANTT_ROW (predecessor));
 
 	priv = arrow->priv;
-
+	
 	if (priv->predecessor) {
 		g_object_remove_weak_pointer (G_OBJECT (priv->predecessor),
 					      (gpointer)&priv->predecessor);
@@ -394,13 +530,13 @@ planner_relation_arrow_set_predecessor (MgRelationArrow *arrow,
 }
 
 void
-planner_relation_arrow_set_successor (MgRelationArrow *arrow,
-				 MgGanttRow      *successor)
+planner_relation_arrow_set_successor (PlannerRelationArrow *arrow,
+				      PlannerGanttRow      *successor)
 {
-	MgRelationArrowPriv *priv; 
+	PlannerRelationArrowPriv *priv; 
 
-	g_return_if_fail (MG_IS_RELATION_ARROW (arrow));
-	g_return_if_fail (MG_IS_GANTT_ROW (successor));
+	g_return_if_fail (PLANNER_IS_RELATION_ARROW (arrow));
+	g_return_if_fail (PLANNER_IS_GANTT_ROW (successor));
 
 	priv = arrow->priv;
 
@@ -430,37 +566,41 @@ planner_relation_arrow_set_successor (MgRelationArrow *arrow,
 	}
 }
 
-MgRelationArrow *
-planner_relation_arrow_new (MgGanttRow *successor, MgGanttRow *predecessor)
+PlannerRelationArrow *
+planner_relation_arrow_new (PlannerGanttRow *successor,
+			    PlannerGanttRow *predecessor,
+			    MrpRelationType  type)
 {
-	MgRelationArrow   *arrow;
-	GnomeCanvasGroup  *root;
+	PlannerRelationArrow *arrow;
+	GnomeCanvasGroup     *root;
 
 	root = gnome_canvas_root (GNOME_CANVAS_ITEM (successor)->canvas);
-
-	arrow = MG_RELATION_ARROW (gnome_canvas_item_new (
-					   root,
-					   MG_TYPE_RELATION_ARROW,
-					   NULL));
 	
+	arrow = PLANNER_RELATION_ARROW (
+		gnome_canvas_item_new (root,
+				       PLANNER_TYPE_RELATION_ARROW,
+				       NULL));
+	
+	arrow->priv->type = type;
+
 	planner_relation_arrow_set_successor (arrow, successor);
 	planner_relation_arrow_set_predecessor (arrow, predecessor);
-	
+
 	return arrow;
 }
 
 static void
-relation_arrow_get_bounds (MgRelationArrow *arrow,
-			   gdouble         *x1,
-			   gdouble         *y1,
-			   gdouble         *x2,
-			   gdouble         *y2)
+relation_arrow_get_bounds (PlannerRelationArrow *arrow,
+			   gdouble              *x1,
+			   gdouble              *y1,
+			   gdouble              *x2,
+			   gdouble              *y2)
 {
-	MgRelationArrowPriv *priv;
-	GnomeCanvasItem     *item;
-	gdouble              wx1, wy1, wx2, wy2;
-	gint                 cx1, cy1, cx2, cy2;
-	gint                 i;
+	PlannerRelationArrowPriv *priv;
+	GnomeCanvasItem          *item;
+	gdouble                   wx1, wy1, wx2, wy2;
+	gint                      cx1, cy1, cx2, cy2;
+	gint                      i;
 
 	item = GNOME_CANVAS_ITEM (arrow);
 	priv = arrow->priv;
@@ -501,12 +641,12 @@ static void
 relation_arrow_update (GnomeCanvasItem *item,
 		       double          *affine,
 		       ArtSVP          *clip_path,
-		       gint              flags)
+		       gint             flags)
 {
-	MgRelationArrow *arrow;
-	gdouble          x1, y1, x2, y2;
+	PlannerRelationArrow *arrow;
+	gdouble               x1, y1, x2, y2;
 
-	arrow = MG_RELATION_ARROW (item);
+	arrow = PLANNER_RELATION_ARROW (item);
 	
 	GNOME_CANVAS_ITEM_CLASS (parent_class)->update (item,
 							affine,
@@ -519,15 +659,15 @@ relation_arrow_update (GnomeCanvasItem *item,
 }
 
 static void
-relation_arrow_setup_arrow (MgArrowDir dir,
-			    GdkPoint   points[4],
-			    gdouble    cx1,
-			    gdouble    cy1,
-			    gdouble    cx2,
-			    gdouble    cy2)
+relation_arrow_setup_arrow (PlannerArrowDir dir,
+			    GdkPoint        points[4],
+			    gdouble         cx1,
+			    gdouble         cy1,
+			    gdouble         cx2,
+			    gdouble         cy2)
 {
 	switch (dir) {
-	case MG_ARROW_DOWN:
+	case PLANNER_ARROW_DOWN:
 		points[0].x = cx2;
 		points[0].y = cy2;
 		points[1].x = cx2 - ARROW_SIZE / 2;
@@ -537,7 +677,7 @@ relation_arrow_setup_arrow (MgArrowDir dir,
 		points[3].x = cx2;
 		points[3].y = cy2 + 1;
 		break;
-	case MG_ARROW_UP:
+	case PLANNER_ARROW_UP:
 		points[0].x = cx2;
 		points[0].y = cy2 + 1;
 		points[1].x = cx2 + ARROW_SIZE / 2;
@@ -547,7 +687,7 @@ relation_arrow_setup_arrow (MgArrowDir dir,
 		points[3].x = cx2 + 1;
 		points[3].y = cy2;
 		break;
-	case MG_ARROW_RIGHT:
+	case PLANNER_ARROW_RIGHT:
 		points[0].x = cx2;
 		points[0].y = cy2;
 		points[1].x = cx2 - ARROW_SIZE;
@@ -557,7 +697,7 @@ relation_arrow_setup_arrow (MgArrowDir dir,
 		points[3].x = cx2;
 		points[3].y = cy2;
 		break;
-	case MG_ARROW_LEFT:
+	case PLANNER_ARROW_LEFT:
 		points[0].x = cx2;
 		points[0].y = cy2;
 		points[1].x = cx2 + (ARROW_SIZE - 0);
@@ -581,22 +721,22 @@ relation_arrow_draw (GnomeCanvasItem *item,
 		     gint             width,
 		     gint             height)
 {
-	MgRelationArrow     *arrow;
-	MgRelationArrowPriv *priv;
+	PlannerRelationArrow     *arrow;
+	PlannerRelationArrowPriv *priv;
 #ifdef USE_AFFINE
-	gdouble              i2c[6];
-	ArtPoint             i1, i2, c1, c2;
+	gdouble                   i2c[6];
+	ArtPoint                  i1, i2, c1, c2;
 #else
-	gdouble              i2w_dx;
-	gdouble              i2w_dy;
-	gdouble              dx1, dy1, dx2, dy2;
+	gdouble                   i2w_dx;
+	gdouble                   i2w_dy;
+	gdouble                   dx1, dy1, dx2, dy2;
 #endif
-	gint                 cx1, cy1, cx2, cy2;
-	GdkGC               *gc;
-	GdkPoint             points[4];
-	gint                 i;
+	gint                      cx1, cy1, cx2, cy2;
+	GdkGC                    *gc;
+	GdkPoint                  points[4];
+	gint                      i;
 	
-	arrow = MG_RELATION_ARROW (item);
+	arrow = PLANNER_RELATION_ARROW (item);
 	priv = arrow->priv;
 
 	gc = gdk_gc_new (drawable);
